#include <cds/gc/dhp.h>
#include <cds/gc/hp.h>
#include <cds/init.h>
#include <cds/intrusive/segmented_queue.h>
#include <iostream>
#include <string>

using namespace std;

namespace ci = cds::intrusive;
typedef cds::gc::HP hp_gc;

struct Foo {
  Foo(int x) : x(x) {}
  int x;
};

// Declare traits for the queue
struct MyTraits : public cds::intrusive::segmented_queue::traits {
  typedef cds::intrusive::segmented_queue::stat<> stat;
  typedef cds::atomicity::item_counter item_counter;
};

// The queue stores the pointers to enqueued items so no special node hooks are
// needed.
// At least, declare the queue type
typedef ci::SegmentedQueue<hp_gc, Foo, MyTraits> MyQueue;

int main() {
  cds::Initialize();

  {
    // Initialize Hazard Pointer singleton
    cds::gc::HP hpGC(128, 8, 128);
    // If main thread uses lock-free containers
    // the main thread should be attached to libcds infrastructure
    cds::threading::Manager::attachThread();

    MyQueue q(1024);
    Foo *f = new Foo(1);
    q.enqueue(*f);
    Foo *res = q.dequeue();
    if (res) {
      cout << "Dequeued " << res->x << "\n";
    } else {
      cout << "Dequeued none\n";
    }
  }

  cds::Terminate();
  return 0;
}
